#include <SDL2/SDL.h>

#include "export.h"
#include "global.h"

void doHandleInput(SDL_Event event) {
  if (event.type == SDL_QUIT) {
    exit(0);
  } else if (event.type == SDL_KEYDOWN) {
    if (event.key.keysym.sym == SDLK_w && SDL_GetModState() & KMOD_CTRL) {
      saveTGA();
    } else if (event.key.keysym.sym == SDLK_q && SDL_GetModState() & KMOD_CTRL) {
      exit(0);
    }
  } else if (event.type == SDL_MOUSEBUTTONDOWN) {
    if (event.button.button == SDL_BUTTON_LEFT) {
      int x = event.button.x;
      int y = event.button.y;
      if (x < FRAME_WIDTH && y < FRAME_HEIGHT) {
        int gridX = x / PIXEL_SIZE;
        int gridY = y / PIXEL_SIZE;
        grid[gridX][gridY] = 4;
      } else if (x >= FRAME_WIDTH && x < SCREEN_WIDTH && y < COLOR_PICKER_WIDTH) {
        struct color selectedColor;

        int gotCP = colorPicker(&selectedColor);
        if (gotCP != 0) {
          fprintf(stderr, "色選択ツールを読込に失敗\n");
        }

        SDL_SetRenderDrawColor(
          renderer,
          selectedColor.red,
          selectedColor.green,
          selectedColor.blue,
          255
        );
      }
    }
  } else if (event.type == SDL_MOUSEMOTION) {
    if (event.motion.state & SDL_BUTTON_LMASK) {
      int x = event.motion.x;
      int y = event.motion.y;
      if (x > FRAME_WIDTH || y > FRAME_HEIGHT) return;
      int gridX = x / PIXEL_SIZE;
      int gridY = y / PIXEL_SIZE;
      grid[gridX][gridY] = 0;
    }
  }
}

void handleInput() {
  SDL_Event event;
  while (SDL_PollEvent(&event)) {
    doHandleInput(event);
  }
}
